//--------------------------------------------------------------------
//
//  Modul crt
//
//  Simuliert den an den Display-Controller angeschlossenen Monitor
//
//--------------------------------------------------------------------

`include "discount_defs.v"

module crt(
  input PIXEL, CSYNC, HSYNC, VSYNC, SNAPSHOT
);

  // Stiehl clock von der Testbench
  // Es waere realistischer, die Clock aus den PIXEL-
  // und Synchronsignalen zurueckzugewinnen. Dies wuerde
  // allerdings mindestens ein Monitorbild mit diskreten
  // einzelnen weissen oder schwarzen Pixeln erfordern,
  // was aber nicht garantiert werden kann
  wire CLK = test_res.CLOCK;

  // Bildpuffer
  reg [2**23-1:0] VBUF;

  // Index in Bildpuffer
  integer i;

  // Spalten und Zeilen
  integer hsize, vsize;

  // Frame ist vollstaendig
  reg FRAME = 0;

  // Laufende Nr. fuer Bilddatei
  integer snapshot_count = 0;

  // Impliziter Zustandsautomat
  always begin
    // neuer Frame (Achtung, nach Reset 'bx)
    while (VSYNC !== 0) @(posedge CLK);
    i = 0; hsize = 0; vsize = 0;
    FRAME = 0;
    // auf ersten HSYNC warten
    // (faengt auch halben Frame,
    // falls nicht mit VSYNC begonnen)
    while (!HSYNC) @(posedge CLK);
    while (HSYNC) @(posedge CLK);
    // Pixel bis zum naechsten HSYNC merken
    while (!VSYNC) begin
      while (!HSYNC) begin
        VBUF[i] = PIXEL;
        i = i + 1;
        @(posedge CLK);
      end
      // HSYNC abwarten
      while (HSYNC) @(posedge CLK);
      // Zeilenbreite einmal merken
      if (!hsize) hsize = i;
      vsize = vsize + 1;
    end
    FRAME = 1;
  end

  // Frame als PBM speichern (SNAPSHOT == 1)
  always @(posedge SNAPSHOT or posedge FRAME) begin: local_SNAPSHOT
    integer i, fd;
    reg [8*256:1] fn;

    // Wenn Bild vollstaendig und SNAPSHOT aktiv
    if (SNAPSHOT && FRAME) begin
      $display("Writing bitmap 'snapshot_%0d.pbm' : hsize=%0d vsize=%0d Pixels=%0d",
               snapshot_count, hsize, vsize, hsize * vsize);
      // Dateinamen lfd. Nr. generieren und Datei oeffnen
      $swrite(fn, "snapshot_%0d.pbm", snapshot_count);
      fd = $fopen(fn, "wb");
      if (!fd) begin
        $write("*** SNAPSHOT: Couldn't open file for writing.\n");
        $finish;
      end
      // PBM Kopfdaten schreiben
      $fwrite(fd, "P1\n%0d %0d\n", hsize, vsize);
      // Bitmap schreiben
      for (i = 0; i < hsize * vsize; i = i + 1)
        $fwrite(fd, "%b ", ~VBUF[i]);
      $fclose(fd);
      snapshot_count = snapshot_count + 1;
    end
  end

endmodule
