//--------------------------------------------------------------------
//
// DISCOUNT: Display-Controller
//
// Stellt einen 8-Bit-Speicher auf einem Video-Monitor monochrom dar
// (Aufloesung 640x192, Bildfrequenz 50 Hz).
//
//--------------------------------------------------------------------

//--------------------------------------------------------------------
//
// Modul discount_res - Version mit variabel programmierbarer Auflosung
//
// Display-Controller (Cathode-Ray-Tube-Controller)
//
//--------------------------------------------------------------------

`include "discount_defs.v"

module discount_res (ADDR, PIXEL, CSYNC, HSYNC, VSYNC,
                     DATA, CLOCK, NRST, PADDR, WE, PDATA);

  // Ausgaenge
  output [`Asz-1:0] ADDR;   // Adresse fuer Bildspeicher
  output            PIXEL,  // Bildpunkt (0: dunkel, 1: hell)
                    CSYNC,  // kombinierte Synchronisation
                            //   (HSYNC und VSYNC)
                    HSYNC,  // horizontale (Zeilen-)Synchronisation
                            //   (1-aktiv)
                    VSYNC;  // vertikale (Bild-)Synchronisation
                            //   (1-aktiv)

  // Eingaenge
  input  [`Dsz-1:0] DATA;   // Daten aus dem Bildspeicher
  input             CLOCK,  // Takt (einphasig)
                    NRST,   // Initialisierung (0-aktiv)
                    WE;     // Write-Enable fuer Programmierbus
  input  [`Asz-1:0] PADDR;  // Programmier-Adressbus

  // Bidirektional
  inout  [`Rsz-1:0] PDATA;  // Programmier-Datenbus

  // Port-Typen
  wire   [`Asz-1:0] ADDR,
                    PADDR;
  wire   [`Dsz-1:0] DATA;
  wire              PIXEL,
                    CSYNC,
                    HSYNC,
                    VSYNC,
                    CLOCK,
                    NRST,
                    WE;
  wire   [`Rsz-1:0] PDATA;

  // interne Variablen
  wire              ADRIE,  // Freigabe: Bilddaten uebernehmen und
                            //   Bildadresse erhoehen
                            //   (address increment enable)
                    NXTAD,  // Bildadresse hochzaehlen
                    NXTLN;  // Zeile hochzaehlen

  //
  // kombinierte Synchronisation
  //
  assign CSYNC = HSYNC ^ VSYNC;

  //
  // Instanzen
  //
  hcount_res HCOUNT (NXTAD, NXTLN, HSYNC, CLOCK, NRST, PADDR, PDATA, WE);
  vcount_res VCOUNT (ADRIE, VSYNC, NXTLN, CLOCK, NRST, PADDR, PDATA, WE);
  memacc_res MEMACC (ADDR,  PIXEL, DATA,  ADRIE, NXTAD,
                     CLOCK, NRST, PADDR, PDATA, WE);

endmodule  // discount_res
